<?php

namespace App\HttpController\Google;

use App\Key\K;
use App\Tools\Proxy;
use App\Tools\ProxyManager;
use App\Tools\Shell;
use App\Tools\System;
use App\Tools\WechatNotice;
use EasySwoole\Http\Message\Status;
use Swlib\SaberGM;
use Swoole\Coroutine;

class Subscribe extends Base
{
    protected function onRequest(?string $action): ?bool
    {
        parent::onRequest($action);
//        if (!current($this->request()->getHeader('bigemap'))&&!$this->request()->getQueryParam('bigemap')){
//            $this->writeJson(500, 'bigemap-server error');
//            return false;
//        }
        $response = $this->response();
        $response->withHeader('Access-Control-Allow-Origin', '*');//允许所有跨域
        $response->withHeader('Access-Control-Allow-Methods', 'GET,POST,PUT,DELETE,OPTIONS');
        $response->withHeader('Access-Control-Allow-Credentials', 'true');
        $response->withHeader('Access-Control-Allow-Headers', 'Content-Type, token,Authorization, bigemap,X-Requested-With');
        if ($this->request()->getMethod() === 'OPTIONS') {
            $response->withStatus(Status::CODE_OK);
            return false;
        }
        return true;
    }
    public function page(){
        //这里是前台页面管理订阅的地址
        $manager = ProxyManager::getInstance();
        $serverData = $manager->getAllServer();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $getData = $this->request()->getQueryParams();
                $search = $getData['search'];
                $search = json_decode($search, true);
                $items = array_filter($serverData, function ($v) use ($search) {
                    $result = [];
                    foreach (['title'] as $vv) {
                        if ($search[$vv] && strpos($v[$vv], $search[$vv]) === false) $result[] = false;
                    }
                    $result = array_unique($result);
                    return !in_array(false, $result);
                });
                $page['pageNumber'] = $getData['pageNumber'] ? intval($getData['pageNumber']) : 1;
                $page['pageSize'] = $getData['pageSize'] ? intval($getData['pageSize']) : 10;
                $page['total'] = count($items);
                $page['subscribe'] = array_slice($items, ($page['pageNumber'] - 1) * $page['pageSize'], $page['pageSize']);
                return $this->jsonSuccess($page);
                break;
            case 'POST':
                $data = $this->getRequestData();
                $serverInfo = Proxy::parseUrl($data);
                if (!$serverInfo) return $this->jsonError('获取订阅失败!');
                $pid = $manager->add('subscribe', $data);
                if (!$pid) return $this->jsonError('保存数据库失败!');
                foreach ($serverInfo as $k => $v) {
                    $manager->add('server_list', ['key' => $k, 'pid' => $pid, 'info' => json_encode($v, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES)]);
                }
                $result = $manager->find('subscribe', ['id' => $pid]);
                return $this->jsonSuccess($result);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('没有此条数据!');
                $manager->save('subscribe', ['id' => $data['id']], $data);
                if ($data['resubscribe']) {
                    //如果重新订阅地址
                    if (!$manager->reSubscribeLink($data['id'])){
                        return $this->jsonError('获取订阅失败!');
                    }
//                    $serverInfo = Proxy::parseUrl($data['url']);
//                    if (!$serverInfo) {
//                        return $this->jsonError('获取订阅失败!');
//                    }
//                    $oldData = $manager->select('server_list', ['pid' => $data['id']], 'key');
//                    //求出已经是过期订阅的服务器
//                    $removeServer = array_diff_key($oldData, $serverInfo);
//                    foreach ($removeServer as $v) {
//                        $manager->delete('server_list', ['id' => $v['id']]);
//                    }
//                    foreach ($serverInfo as $k => $v) {
//                        if (!$oldData[$k]) {
//                            //本地没有保存这个服务器，添加新的
//                            $addData = ['key' => $k, 'pid' => $data['id'], 'info' => json_encode($v, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES)];
//                            $manager->add('server_list', $addData);
//                        } else {
//                            //更新数据
//                            $oldData[$k]['info'] = json_encode($v, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
//                            $manager->save('server_list', ['id' => $oldData[$k]['id']], $oldData[$k]);
//                        }
//                    }
                }
                $allServer=ProxyManager::getInstance()->getAllServer();
                foreach ($allServer as $v){
                    if ($v['id']==$data['id']){
                        //由于要获取当前有多少个服务器在使用这个代理 ，所以这里通过这个办法来获取的
                        $newData=$v;
                        break;
                    }
                }
//                $newData = $manager->find('subscribe', ['id' => $data['id']]);
                return $this->jsonSuccess($newData);
                break;
            case 'DELETE':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('没有此条数据!');
                if (!$manager->delete('server_list', ['pid' => $data['id']])) return $this->jsonError('删除子数据失败!');
                if (!$manager->delete('subscribe', ['id' => $data['id']])) return $this->jsonError('删除订阅数据失败!');
                return $this->jsonSuccess('删除成功!');
                break;
        }
    }

    public function index()
    {
        $manager = ProxyManager::getInstance();
        $serverData = $manager->getAllServer();
        switch ($this->request()->getMethod()) {
            case 'GET':
                //这里是所有的服务器在获取启动代理的获取接口
                foreach ($serverData as $k => $v) {
                    //如果没有启用，或者是超过了限制数，那么不返回没有启用或者是超限的订阅
                    if ($v['enable']!=1||$v['in_use'] >= $v['limit']) unset($serverData[$k]);
                    if (!$this->request()->getQueryParam('all')){
                        //如果不是要全部的代理，那么要去掉一些不用的代理
                        if ($v['enable']!=1||$v['in_use'] >= $v['limit']) unset($serverData[$k]);
                    }
                }
                return $this->jsonSuccess(array_values($serverData));
                break;
        }
    }

    public function traffic()
    {
        //这里是定期检查流量的使用情况然后更新数据的
        $manager = ProxyManager::getInstance();
        $data = $this->getRequestData();
        if (!$data['id']) return $this->jsonError('no id');
        $manager->save('subscribe', ['id' => $data['id']], ['left' => $data['left'], 'left_time' => $data['left_time']]);
        $serverData = $manager->find('subscribe', ['id' => $data['id']]);
        //如果当前的代理没有启用，那么不会提交数据
        if ($serverData['enable']!='1') return;
        if (time()-$serverData['resubscribe_time']>0){
            //距离上次更新订阅已经超过了2天，那么要重新更新订阅的服务器列表
            $manager->reSubscribeLink($data['id']);
        }
        $noticeData['content'] = $serverData['title'];
        if ($serverData['deadline'] - time() <= 0 && $serverData['deadline']) {
            $noticeData['content'] .= "\r\n".'过期时间：'.date('Y-m-d',$serverData['deadline']);
            $noticeData['action'] = 'subscribe_expire';
            WechatNotice::notice($noticeData);
        }
        $low = false;
        switch (strtoupper(substr($data['left'], -2))) {
            case 'GB':
                if (intval($data['left']) < 10) $low = true;
                break;
            case 'MB':
                $low = true;
                break;
        }
        $noticeData['content'] = $serverData['title'];
        if (intval($data['left']) < 0 || $low) {
            $noticeData['action'] = 'subscribe_low_traffic';
            $noticeData['key']=md5($serverData['title']);
            WechatNotice::notice($noticeData);
        }
        return $this->jsonSuccess('ok');
    }
    public function test(){
        //此方法主要是用来检查当前的代理能不能访问，或者当前的订阅能不能在外网订阅
        $id=$this->request()->getQueryParam('id');
        $manager=ProxyManager::getInstance();
        $serverData=$manager->find('server_list',['id'=>$id]);
        if (!$serverData) return $this->jsonError('没有此代理!');
        $info=json_decode($serverData['info'],true);
        if (!$info) return  $this->jsonError('代理信息解析不正确');
        $template = Proxy::getBaseTemplate();
        if (!$template) return  $this->jsonError('获取模版出错!');
        $proxyPort=50002;
        $templateClash = Proxy::getBaseTemplate(K::PROXY_TYPE_CLASH);
        if (!$templateClash) return  $this->jsonError('获取模版Clash出错!');
        if (!Proxy::startProxy($template, $info, $proxyPort,['base_type'=>$serverData['base_type'],'template'=>$templateClash]))return  $this->jsonError('启用代理失败!');
        Coroutine::sleep(3);
        try {
            $ip=SaberGM::get(K::MANAGE_SERVER_ADDR_9502.'/google/index/ip',['timeout'=>2,'proxy'=>'socks5://127.0.0.1:'.$proxyPort]);
            $ip=trim($ip->getBody()->getContents());
            if (filter_var($ip,FILTER_VALIDATE_IP)){
                $manager->save('server_list',['id'=>$id],['ip'=>$ip,'update_time'=>time()]);
                $serverData['ip']=$ip;
                $serverData['update_time']=time();
            }
            Coroutine::sleep(1);
            Shell::shellKill('v2ray/config'.$proxyPort);
        }catch (\Throwable $exception){
            return  $this->jsonError($exception->getMessage());
        }
        return  $this->jsonSuccess($serverData);
    }
    public function server()
    {
        $proxyManager = ProxyManager::getInstance();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $servers = $proxyManager->getAllServer();
                $items = [];
                foreach ($servers as $v) {
                    $v['children'] = array_map(function ($v) {
                        $v['info'] = json_decode($v['info'], true);
                        $v['ps'] = $v['info']['ps'];
                        return $v;
                    }, $v['children']);
                    //增加这个属性方便前台来获取所有的订阅方便还原数据
                    $page['mapping'][]=['value'=>$v['id'],'label'=>$v['title']];
                    $items = array_merge($items, $v['children']);
                }
                $getData = $this->request()->getQueryParams();
                $search = $this->request()->getQueryParam('search');
                $search = json_decode($search, true);
                $items = array_filter($items, function ($v) use ($search) {
                    $result = [];
                    $v['info'] = json_decode($v['info']);
                    foreach (['ip', 'server_id', 'ps', 'pid','forbidden_server_id'] as $vv) {
                        if ($search[$vv] && strpos($v[$vv].'', $search[$vv].'') === false) $result[] = false;
                    }
                    foreach (['status'] as $vv) {
                        if (isset($search[$vv])&&$search[$vv]!=='' && $v[$vv]!=$search[$vv]) $result[] = false;
                    }
                    $result = array_unique($result);
                    return !in_array(false, $result);
                });
                $page['pageNumber'] = $getData['pageNumber'] ? intval($getData['pageNumber']) : 1;
                $page['pageSize'] = $getData['pageSize'] ? intval($getData['pageSize']) : 10;
                $page['total'] = count($items);
                $page['server'] = array_slice($items, ($page['pageNumber'] - 1) * $page['pageSize'], $page['pageSize']);
                return $this->jsonSuccess($page);
                break;
            case 'DELETE':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('没有此条数据!');
                $result = $proxyManager->delete('server_list', ['id' => $data['id']]);
                if (!$result) return $this->jsonError('删除失败!');
                return $this->jsonSuccess('删除成功!');
                break;
        }
    }

}
